#!/usr/bin/php
<?php
define('DATA_DIR',"/tmp/esia_scan");

include_once('/usr/local/esia/plugins/lib/php/scan.lib.php');

define('SCRIPT_NAME',basename($_SERVER['SCRIPT_NAME']));
define('EXIT_OK',0);
define('EXIT_ERROR',1);


function print_help($exit=null){
	echo "Used to manage (ADD/GET/DELETE) scan tasks.\n";
	echo "\n";
	print_usage();
	echo "\n";
	echo "Actions :\n";
	echo " -A (--add)          add a scan task in the scanner. (-i specify ONE task and not a list).\n";
	echo " -L (--list)         Get a list of all scan. Can be filtered with a list of status.\n";
	echo " -G (--get)          Get all the scan details in the -i list.\n";
	echo " -P (--partial)	   Get piece of the scan details (use with -p and -l option)\n";
	echo " -D (--delete)       Delete all the IDs in the -i list.\n";
	echo "\n";
	echo "Options :\n";
	echo " -i (--id=LIST)      Comma separated ids list.\n";
	echo " -a (--addr=LIST)    Comma separated address list (10.7.0.1,10.7.0.2,10.7.0.3 or 10.7.0.1-10.7.0.3 or 10.7.0.0/24).\n";
	echo " -t (--type=TYPE)    Scan TYPE : ICMP(default), SNMP.\n";
	echo " -o (--opts=OPTIONS) Options needed for scan. Null for ICMP. Community list for SNMP\n";
	echo " -s (--state=LIST)   1 or multiples scan status (pending,processing,terminated,error) comma separated.\n";
	echo " -p (--part=INT)     Piece of the result to get (use with -P)\n";
	echo " -l (--lenght=INT)   Size of the piece to get (use with -P)\n";
	echo "\n";
	echo " -h (--help)         Print help.\n";

	if($exit!=null) exit($exit);
}

function print_usage($exit=null){
	echo "Usage : \n";
	echo ' '.SCRIPT_NAME." -A [-i <ID>] -a <ADDR> [-t <TYPE>] [-o <OPTS>]\n";
	echo ' '.SCRIPT_NAME." -L [-s <STATUS>]\n";
	echo ' '.SCRIPT_NAME." -G -i <IDS>\n";
	echo ' '.SCRIPT_NAME." -D -i <IDS>\n";
	echo ' '.SCRIPT_NAME." -h\n";

	if($exit!=null) exit($exit);
}

function exit_error($msg,$print_usage=false){
	echo "ERROR : $msg\n";
	if($print_usage) print_usage(EXIT_ERROR);
	exit(EXIT_ERROR);
}

function getParam($options,$name1,$name2,$default=null){
	if( isset( $options[$name1] ) ) return $options[$name1];
	if( isset( $options[$name2] ) ) return $options[$name2];
	return $default;
}

function add_task($options){
	$id     = getParam($options,'i','id',time());
	$addr   = getParam($options,'a','addr');
	$type   = getParam($options,'t','type','icmp');
	$opts   = getParam($options,'o','opts');

	if (is_string($opts))
	{
			$tmp = json_decode($opts,true);
	}
	if( empty($tmp) || !is_array($tmp) )
	{
		if( !is_array($opts) )
		{
				$opts = array($opts);
		}
		$tmp = array();
		foreach($opts as $opt)
		{
				$exp = explode(':',$opt);
				$key = $exp[0];
				if( !isset($exp[1]) )
				{
						$key = 'communities';
						$value = $exp[0];
				}
				else
				{
						$value = $exp[1];
				}
				$tmp[$key]=$value;
		}
		$opts = json_encode($tmp);
	}


	if( $addr == null ) exit_error("You must specify addresses for the -A (--add) option",true);
	if( $id == null ) $id=time();
	else if( count(explode(',',$id))  > 1 ) exit_error("You must specify one ID for the -A (--add) option. Not a list",true);
	if( !is_numeric($id) ) exit_error("The id must be an integer.",true);

	$array["status"]="pending";
	$array["addresses"]=$options["a"];
	if( $type != null ) $array["type"]=$type;
	if( $opts != null ) $array["options"]=$opts;

    @mkdir(DATA_DIR);
	$json=json_encode($array,JSON_PRETTY_PRINT);
	file_put_contents(DATA_DIR.'/'.$id,$json."\n");
	echo "OK: Task send\n";
}

$longopts = array(
		'add','list','get','partial','delete','help',
		'id:','addr:','type:','opts:','state:','part:','length'
);
$shortopts = 'ALGPDhi:a:t:o:s:p:l:';

$options = getopt($shortopts,$longopts);
if(      isset($options['h'])   || isset($options['help']))     print_help(EXIT_OK);
else if( isset($options['A'])   || isset($options['add']))      add_task($options);
else if( isset($options['L'])   || isset($options['list']))     list_tasks($options);
else if( isset($options['G'])   || isset($options['get']))      get_tasks($options);
else if( isset($options['P'])	|| isset($options['partial']))  get_partial($options);
else if( isset($options['D'])   || isset($options['delete']))   remove_tasks($options);
else exit_error('You must specify action.',true);
?>
